# Developing Extensions Tutorial
JupyterLab is a modern interactive development environment from Project Jupyter for scientific exploration. Users can install or write third-party extensions to customize or extend JupyterLab, enabling them to, for example, view custom file formats, interact with external services, or display their data (interactive maps, tables, plots, etc.). In this tutorial, attendees will be guided through the process of creating a basic extension and taught about the core components that power JupyterLab. 

## Prerequisites
* Python >= 3.7
* Node.js >= 12 (LTS version preferred)
* VSCode, download from [here](https://code.visualstudio.com/)
* (Optional) Note: We'll be using conda as a virtual environment, see [here](https://docs.conda.io/projects/conda/en/latest/user-guide/install/index.html) for instructions on installation of conda

## Tutorial Code
You can find example code in the [examples](https://github.com/marthacryan/developing-extensions-tutorial/tree/main/examples) directory.

## Agenda
* Introduction (5 minutes):
  * Meet the instructors
* Exploring extensions (15 minutes) (Martha)
  * What are extensions?
  * Example Extensions
  * Installing prebuilt vs source
  * Good documentation for developing extensions.
* Anatomy of an extension (Alex)
  * JupyterLab extension (20 minutes + 30 minute exercise)
* Break: 10 minutes
* What to do when things go wrong? (30 minutes + 30 minute exercise) (Piyush)
  * Debugging tutorial
* Working on your own extension (semi-guided on an extension you would like to create (45 minutes + break?)
  * Jupyter Server extension
  * Theme extension
  * Whatever you wanted to start on - or pick from [here](https://github.com/jupyterlab/jupyterlab/labels/tag%3AExtension%20Idea)
* Q & A / Show and Tell (30 minutes)

## Slides 
**Note**: Slides can be found [here](https://docs.google.com/presentation/d/1Sg2gdaniTo6IPwxgW9C2RyuJHTCJGm30PkyaKIpB0g4/edit?usp=sharing) while in development, but we'll move them into the slides directory when finalized

## About the Instructors
* Martha and Alex are software engineers at the IBM Center for Open Source Data and AI Technologies. They are both contributors on [JupyterLab](https://github.com/jupyterlab/jupyterlab) and [Elyra](https://github.com/elyra-ai/elyra).

* Piyush is a software engineer at Amazon Web Services working on [JupyterLab](https://github.com/jupyterlab/jupyterlab).

